package domaine.temps;

import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.Test;

import domaine.temps.Heure;
import domaine.temps.HeureV1;
import domaine.temps.HeureV2;
import domaine.temps.MauvaisFormatDHeure;
import domaine.temps.MauvaiseValeurDesHeures;
import domaine.temps.MauvaiseValeurDesMinutes;

class TestHeure {

	@Test
	void testNouvelleHeureParDefaut() {
		Heure h;
		h = new HeureV1();
		assertEquals(12, h.heures());
		assertEquals(0, h.minutes());
	}

	@Test
	void testNouvelleHeureParEntiers1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Heure h;
		h = new HeureV1(9, 15);
		assertEquals(9, h.heures());
		assertEquals(15, h.minutes());
	}

	@Test
	void testNouvelleHeureParEntiers2() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Exception e = assertThrows(MauvaiseValeurDesHeures.class, () -> new HeureV1(24, 15));
		assertEquals("valeur incorrecte des heures (24) pour une heure", e.getMessage());
	}

	@Test
	void testNouvelleHeureParEntiers3() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Exception e = assertThrows(MauvaiseValeurDesMinutes.class, () -> new HeureV1(8, 60));
		assertEquals("valeur incorrecte des minutes (60) pour une heure", e.getMessage());
	}

	@Test
	void testNouvelleHeureParTexte1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		Heure h;
		h = new HeureV1("11:46");
		assertEquals(11, h.heures());
		assertEquals(46, h.minutes());
		h = new HeureV1("7:30");
		assertEquals(7, h.heures());
		assertEquals(30, h.minutes());
	}

	@Test
	void testNouvelleHeureParTexte2() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		Exception e = assertThrows(MauvaisFormatDHeure.class, () -> new HeureV1("midi"));
		assertEquals("texte incorrect (midi) pour une heure", e.getMessage());
	}

	@Test
	void testNouvelleHeureParTexte3() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		assertThrows(MauvaiseValeurDesHeures.class, () -> new HeureV1("25:05"));
	}

	@Test
	void testNouvelleHeureParTexte4() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		assertThrows(MauvaiseValeurDesMinutes.class, () -> new HeureV1("20:60"));
	}

	@Test
	void testMinutesPlus1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		Heure h;
		h = new HeureV1(10, 10);
		h.minutesPlus1();
		assertEquals(11, h.minutes());
		h = new HeureV1(11, 59);
		h.minutesPlus1();
		assertEquals(0, h.minutes());
		assertEquals(12, h.heures());
	}

	@Test
	void testHeuresPlus1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, MauvaisFormatDHeure {
		Heure h;
		h = new HeureV1(5, 0);
		h.heuresPlus1();
		assertEquals(6, h.heures());
		assertEquals(0, h.minutes());
		h = new HeureV1(23, 15);
		h.heuresPlus1();
		assertEquals(0, h.heures());
		assertEquals(15, h.minutes());
	}

	@Test
	void testMinutesMoins1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Heure h;
		h = new HeureV1(16, 31);
		h.minutesMoins1();
		assertEquals(30, h.minutes());
		assertEquals(16, h.heures());
		h = new HeureV1(21, 0);
		h.minutesMoins1();
		assertEquals(59, h.minutes());
		assertEquals(20, h.heures());
	}

	@Test
	void testHeuresMoins1() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Heure h;
		h = new HeureV1(7, 50);
		h.heuresMoins1();
		assertEquals(6, h.heures());
		assertEquals(50, h.minutes());
		h = new HeureV1(0, 17);
		h.heuresMoins1();
		assertEquals(23, h.heures());
		assertEquals(17, h.minutes());
	}

	@Test
	public void testToString() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Heure h;
		h = new HeureV1(6, 35);
		assertEquals("06:35", h.toString());
		h = new HeureV1(11, 5);
		assertEquals("11:05", h.toString());
	}

	@Test
	public void testComparaisons() throws MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
		Heure h1, h2, h3;
		h1 = new HeureV1(6, 35);
		h2 = new HeureV1(11, 5);
		h3 = new HeureV2(11, 5);
		assertTrue(h1.compareTo(h2) < 0);
		assertTrue(h2.compareTo(h1) > 0);
		assertTrue(h2.compareTo(h3) == 0);
	}
}
